################################################################################
##                      CONFIGURATION REGISTER SETUP                          ##
################################################################################
# The folowing command will configure the driver for AD7699
# Bit 0 Read Back the CFG register (0 - Read back current config, 1 - Do not 
# 		read back contents of CFG register)
# Bit [2:1] Channel sequencer. Allows for scanning channels in IN0 to IN[7:0]
# Bit [5:3] Reference/buffer selection. Selection of intenral, external, and 
#		external buffered references, and enabling of the tempreture sensor.
# Bit 6 Select bandwidth for low-pass filter.
# Bit [9:7] Input channel selection in binary fashion 
# Bit [12:10] Input channel configuration.
# Bit 13 Configuration Update (0 - Keep current configuration, 1 - Overwrite 
#		contents of register)
# NOTE The current revision of the driver does not suport the configuration 
# register readback 
# More detailed description can find under ../EvalBoardFPGA/Docs/AD7699.pdf

set CFG(RdBack)     1 
set CFG(Seq)        0 
set CFG(RefBuff)    7 
set CFG(BandW)      1 
set CFG(InX)        0 
set CFG(INcC)       2 
set CFG(CfgEn)      1 

# end of configuration register setup
set command 0
# Compose command
foreach {field value} [array get CFG] {
    switch $field {
        "RdBack"    {set command [expr $command + $value]}
        "Seq"       {set command [expr $command + [expr $value << 1]]}
        "RefBuff"   {set command [expr $command + [expr $value << 3]]}
        "BandW"     {set command [expr $command + [expr $value << 6]]}
        "InX"       {set command [expr $command + [expr $value << 7]]}
        "INcC"      {set command [expr $command + [expr $value << 10]]}
        "CfgEn"     {set command [expr $command + [expr $value << 13]]}    
    }
}

#Data acquisition

puts "Acquiring data ..."
set startacq 0
set mm [ lindex [ get_service_paths master ] 0 ]
close_service master $mm
open_service master $mm

puts "Connected to device ..."

# Set up the configuration register
master_write_32 $mm 0x4a0018 $command

set startacq [master_read_32 $mm 0x4a0000 1]
set point [master_read_32 $mm 0x4a0014 1 ]
master_write_32 $mm $point 1

# Start the acquisition
while {$startacq == 0} {
    set startacq [master_read_32 $mm 0x4a0000 1]
}

puts "Acquisition has been started..."
#maximum number of data bytes that can be acquired is 1048576
set data [ master_read_memory $mm 0x4e0000 524288 ]

master_write_32 $mm 0x4a0004 1

puts "Converting acquired data to 16 bit values"

for { set i 0 } { $i < [ llength $data ] } { incr i 2} {
    set a [ lindex $data $i ]
    set b [ lindex $data [ expr { $i + 1 } ] ]
    lappend newData [ expr  { ( $b << 8 ) + $a } ]
}

puts "Processing data as two's complement"
set final " "
if { [expr $CFG(Seq) == 0] } {
    lappend final "Channel$CFG(InX)\n"
	for { set i 0 } { $i < [ llength $newData ] } { incr i} {
    	set a [ lindex $newData $i ]
    	set decimal [expr $a]
    	if { $decimal > 32767 } {
        	set decimal [expr $decimal - 65535 ]
     	}
    	lappend final [expr $decimal ]\n
    }
} else {
    lappend final "Channel0, Channel1, Channel2, Channel3, Channel4, Channel5, Channel6, Channel7\n"
	for { set i 0 } { $i < [ llength $newData ] } { incr i 8} {
        set a [ lindex $newData $i ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 1] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 2] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData  [expr $i + 3] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 4] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 5] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 6] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ],

        set a [ lindex $newData [expr $i + 7] ]
        set decimal [expr $a]
        if { $decimal > 32767 } {
            set decimal [expr $decimal - 65535 ]
        }
        lappend final [expr $decimal ]\n
    }
}

unset data
unset newData

master_write_32 $mm 0x4a0008 1

puts "Moving data into file Acquisition.csv"
# NOTE : If the file is allready exist it will be overwritten, save the 
# previouse acquisition result if needed

set openFile 0

while { $openFile == 0 } {
    if [catch { open "Acquisition.csv" "w+" } fid ] {
        puts stderr "Error opening the file. Please make sure the file is not \
        	opened by another program"
        after 1000
        puts "Trying again..."
    } else {
        set fd $fid
        set openFile 1
        puts "File opened"
    }
}

puts -nonewline $fd [ join $final ]
close $fd
master_write_32 $mm 0x4a000C 1
unset final
puts "Iteration complete"

close_service master $mm

